from dataclasses import dataclass
from typing import NewType, Optional, List, Literal, Dict, Any
from .project import ProjectId
from .batch import BatchId
from .pagination import PaginationFilters

TaskId = NewType("TaskId", str)


TaskStatus = Literal["pending", "complete", "configuring"]
TaskOptionalFields = Literal["metrics", "jobs", "data", "data.mask"]
TaskCompatibilityMode = Literal["kili", "lab"]

@dataclass
class TaskPayload(dict):
    """Task payload for task creation

    
    Args:
        name: Name of the task (should be unique in a batch)
        resources: Path or URL of a resource (ex.: "./images/image1.jpg", "https:://domain.com/image1.jpg")
        data: Initial data for the task (depends on project type and configuration)
    """
    #Inherit from dict to make it JSON serializable

    name: str
    data: Dict[str, Any]
    resources: Optional[List[str]] = None
    

    def __init__(self, name:str, resources : Optional[List[str]], data:  Dict[str, Any]):
        self.name = name
        self.resources = resources
        self.data = data
        dict.__init__(self, name=name, resources=resources, data=data)

@dataclass
class TaskFilters(PaginationFilters):
    """Task filters for running a task search."""

    batch_id_in: Optional[List[BatchId]] = None
    project_id: Optional[ProjectId] = None
    status_in: Optional[List[TaskStatus]] = None
    task_id_in: Optional[List[TaskId]] = None
    name_in: Optional[List[str]] = None
    name_like: Optional[str] = None
    created_at_gt: Optional[str] = None
    created_at_gte: Optional[str] = None
    created_at_lt: Optional[str] = None
    created_at_lte: Optional[str] = None
    updated_at_gt: Optional[str] = None
    updated_at_gte: Optional[str] = None
    updated_at_lt: Optional[str] = None
    updated_at_lte: Optional[str] = None
    optional_fields: Optional[List[TaskOptionalFields]] = None



from typing import Generator, Dict, List, Union, Optional
import time
import json
from isahitlab.api.base import BaseApi
from isahitlab.domain.task import TaskFilters
from isahitlab.domain.batch import BatchId
from isahitlab.helpers.list import divide_chunks
from .mappers import task_filters_mapper
from ..helpers import get_response_json, log_raise_for_status, Paginated

class TaskApi(BaseApi):
    """Task API Calls"""

    def get_all_tasks(self, filters: TaskFilters) -> Generator[tuple[Dict, int, int], None, None] :
        """Get tasks"""

        page = 1
        limit = 10

        filters.pagination = True
        filters.limit = limit

        while True:
            filters.page = page

            paginated: Paginated = self._get_tasks_page(filters)
            docs = paginated['docs']
            page = paginated['nextPage']

            # Count already loaded
            totalDocs = paginated['totalDocs']
            loaded = (paginated['page'] - 1) * paginated['limit'] + len(docs)

            yield (docs, loaded, totalDocs)

            if not paginated['hasNextPage']:
                break

            time.sleep(0.5)

    def get_existing_task_names(self, batch_id : BatchId, names: List[str]) -> Dict[str, List[str]]:
        """Get existing task names by chunk"""
        chunks = divide_chunks(names, 500)

        duplicates = []

        for chunk in chunks:
            res = self._get_existing_task_names_chunk(batch_id, chunk)
            if len(res["duplicate"]) > 0:
                duplicates = duplicates + res["duplicate"]

        return duplicates


    def create_task(self, batch_id : BatchId, name : str, resources : List[Union[str, Dict]], data_id : Union[Dict, None], grant_resources: Optional[str] = None, grant_data: Optional[str] = None) -> Dict:
        """Create task"""
        data = { 
            "name": name,
            "resources" : resources,
            "dataId" : data_id,
            "grantResources": grant_resources,
            "grantData": grant_data,
        }

        created = self._http_client.post('api/task-manager/batches/{}/tasks'.format(batch_id), json=data)

        log_raise_for_status(created)

        return get_response_json(created)

    def _get_existing_task_names_chunk(self, batch_id : BatchId, names: List[str]) -> Dict[str, List[str]]:
        """Get existing tasks
        
            Args:
                names: List of name to check (limited to 500)
           """

        existings = self._http_client.patch('api/task-manager/batches/{}/tasks-duplicate'.format(batch_id), json={ "names" : names })

        log_raise_for_status(existings)

        return get_response_json(existings)

    
    def _get_tasks_page(self, filters: TaskFilters) -> Paginated :
        """Get tasks page"""
        
        tasks = self._http_client.get('api/task-manager/tasks', params=task_filters_mapper(filters))
        
        log_raise_for_status(tasks)
        
        return get_response_json(tasks)
    


import json
import logging
import uuid
from typing import Dict, Iterable, List, Optional, Union

from isahitlab.api.dataset.api import DatasetApi
from isahitlab.domain.batch import BatchId
from isahitlab.domain.dataset import DatasetId, FilePayload
from isahitlab.domain.project import ProjectId
from isahitlab.helpers.list import list_duplicates, remove_duplicates
from isahitlab.helpers.resource import is_resource, is_url
from isahitlab.operations.base import BaseAction
from tqdm import tqdm
from typeguard import typechecked

logger = logging.getLogger('isahitlab.append_to_dataset')


class AppendToDatasetOperation(BaseAction):
    """Upload files to a dataset

    """


    _dataset_api : DatasetApi

    @typechecked
    def run(
        self,
        project_id: ProjectId,
        dataset_id: BatchId,
        files: List[FilePayload],
        disable_progress_bar: Optional[bool] = False,
    ) -> None:
        """ Add resources to a dataset
        
            project_id: ID of the project
            dataset_id: ID of the dataset_id
                You can also provide a batch_id to use the dataset linked to the batch
            batch_id: ID of the batch
            files : list of the file to create, str or Dict -> FilePayload(path : str, file : str)
            disable_progress_bar: Disable the progress bar display
        """
        self._dataset_api = DatasetApi(self._http_client)
        nb_files = len(files)
        
        with tqdm(total=nb_files,  disable=disable_progress_bar, desc="Uploading files... ") as loader:
            for file in files:
                self._upload_file(file.file, file.path, dataset_id, project_id)
                loader.update(1)
            
    
    def _upload_file(self, resource: str, folder: str, dataset_id: DatasetId, project_id: ProjectId) -> Dict:
        with open(resource, 'rb') as f:
            file_info = self._dataset_api.upload_file(dataset_id=dataset_id, project_id=project_id, file=f, folder=folder)
        return file_info

